#!/usr/bin/env python3
# This file is a part of marzer/poxy and is subject to the the terms of the MIT license.
# Copyright (c) Mark Gillard <mark.gillard@outlook.com.au>
# See https://github.com/marzer/poxy/blob/master/LICENSE for the full license text.
# SPDX-License-Identifier: MIT
"""
Functions and classes for working with m.css.
"""

import shutil
from .utils import *
from . import dirs



def update_bundled_install(source_root: Path):
	assert source_root is not None

	source_root = coerce_path(source_root).resolve()
	assert_existing_directory(source_root)
	assert_existing_file(Path(source_root, r'documentation/doxygen.py'))
	if dirs.MCSS == source_root:
		raise Exception(r'm.css source path may not be the same as the internal destination.')

	# delete existing m.css
	if dirs.MCSS.exists():
		delete_directory(dirs.MCSS, logger=True)

	# copy new one
	print(rf'Updating bundled m.css from {source_root}')
	shutil.copytree(
		source_root,
		dirs.MCSS,
		symlinks=False,
		dirs_exist_ok=True,
		ignore=shutil.ignore_patterns(
		r'.git*',  #
		r'.editor*',
		r'.circleci*',
		r'.coverage*',
		r'.istanbul*',
		r'*.idx',
		r'*.pyc',
		r'*.compiled.css',
		r'__pycache__*',
		r'artwork*',
		r'circleci*',
		r'test_doxygen*',
		r'test_python*',
		r'pelican-theme*',
		r'pygments-*.py',
		r'postprocess.sh',
		r'postprocess.py',
		r'm-*dark.css',  # the m.css themes have local copies in data/ for *reasons*
		r'm-*light.css'
		)
	)

	# delete unwanted files
	# todo: figure how to do this as a filter, instead of copying -> deleting
	for folder in (
		r'doc',  #
		r'documentation/test',
		r'documentation/templates/python',
		r'package',
		r'plugins/m/test',
		r'site',
	):
		delete_directory(Path(dirs.MCSS, folder), logger=True)


#!/usr/bin/env python3
# This file is a part of marzer/poxy and is subject to the the terms of the MIT license.
# Copyright (c) Mark Gillard <mark.gillard@outlook.com.au>
# See https://github.com/marzer/poxy/blob/master/LICENSE for the full license text.
# SPDX-License-Identifier: MIT
"""
Functions and classes for working with various source control repositories.
"""

from .utils import *

RX_REPO_PAIR = re.compile(r"""\s*([a-zA-Z0-9_+.-]+)\s*[/\\:,;|]\s*([a-zA-Z0-9_+.-]+)\s*""")



def extract_user_and_repo(s) -> typing.Tuple[str, str]:
	assert s is not None
	s = str(s)
	global RX_REPO_PAIR
	m = RX_REPO_PAIR.fullmatch(s)
	if not m:
		raise Error(rf'could not extract user-repository pair from "{s}"')
	return (m[1], m[2])



# =======================================================================================================================
# GitHub
# =======================================================================================================================



class GitHub(object):

	KEY = r'github'

	def __init__(self, user_and_repo: str):
		urp = extract_user_and_repo(user_and_repo)
		self.user = urp[0]
		self.repository = urp[1]
		self.uri = rf'https://github.com/{self.user}/{self.repository}'
		self.issues_uri = rf'{self.uri}/issues'
		self.pull_requests_uri = rf'{self.uri}/pulls'
		self.releases_uri = rf'{self.uri}/releases'
		self.release_badge_uri = rf'https://img.shields.io/github/v/release/{self.user}/{self.repository}?style=flat-square',
		self.icon_filename = rf'poxy-icon-{GitHub.KEY}.svg'

	def __bool__(self) -> bool:
		return True

	def __str__(self) -> str:
		return rf'{self.user}/{self.repository}'

	def make_user_uri(self, username) -> str:
		assert username is not None
		return rf'https://github.com/{str(username).strip()}'

	def make_issue_uri(self, key) -> str:
		assert key is not None
		return rf'{self.issues_uri}/{str(key).strip()}'

	def make_pull_request_uri(self, key) -> str:
		assert key is not None
		return rf'{self.pull_requests_uri}/{str(key).strip()}'



# =======================================================================================================================
# GitLab
# =======================================================================================================================



class GitLab(object):

	KEY = r'gitlab'

	def __init__(self, user_and_repo: str):
		urp = extract_user_and_repo(user_and_repo)
		self.user = urp[0]
		self.repository = urp[1]
		self.uri = rf'https://gitlab.com/{self.user}/{self.repository}'
		self.issues_uri = rf'{self.uri}/-/issues'
		self.pull_requests_uri = rf'{self.uri}/-/merge_requests'
		self.releases_uri = rf'{self.uri}/-/releases'
		self.release_badge_uri = None  # todo
		self.icon_filename = rf'poxy-icon-{GitLab.KEY}.svg'

	def __bool__(self) -> bool:
		return True

	def __str__(self) -> str:
		return rf'{self.user}/{self.repository}'

	def make_user_uri(self, username) -> str:
		assert username is not None
		return rf'https://gitlab.com/{str(username).strip()}'

	def make_issue_uri(self, key) -> str:
		assert key is not None
		return rf'{self.issues_uri}/{str(key).strip()}'

	def make_pull_request_uri(self, key) -> str:
		assert key is not None
		return rf'{self.pull_requests_uri}/{str(key).strip()}'



# =======================================================================================================================
# all repo types
# =======================================================================================================================

TYPES = (GitHub, GitLab)
KEYS = tuple([t.KEY for t in TYPES])



#!/usr/bin/env python3
# This file is a part of marzer/poxy and is subject to the the terms of the MIT license.
# Copyright (c) Mark Gillard <mark.gillard@outlook.com.au>
# See https://github.com/marzer/poxy/blob/master/LICENSE for the full license text.
# SPDX-License-Identifier: MIT
"""
"XML utilities" - Helpers for working with XML using lxml.

(I wanted to call this module simply "xml" but that breaks BeautifulSoup for some reason)
"""

from lxml import etree
from .utils import *



def create_parser(remove_blank_text=False, **kwargs) -> etree.XMLParser:
	return etree.XMLParser(
		encoding=r'utf-8',  #
		remove_blank_text=remove_blank_text,
		recover=True,
		remove_comments=True,
		remove_pis=True,
		ns_clean=True,
		**kwargs
	)



DEFAULT_PARSER = create_parser()



def make_child(parent, tag_name: str, **attrs):
	assert parent is not None
	assert tag_name is not None
	assert tag_name
	return etree.SubElement(parent, tag_name, attrib=attrs)



def read(source: typing.Union[str, bytes, Path], parser=None, logger=None):
	assert source is not None
	assert source
	assert isinstance(source, (str, bytes, Path))

	if parser is None:
		parser = DEFAULT_PARSER

	if isinstance(source, Path):
		source = read_all_text_from_file(source, logger=logger)
	if isinstance(source, str):
		source = source.encode(r'utf-8')
	return etree.fromstring(source, parser=parser)



ElementTypes = typing.Union[etree.ElementBase, etree._Element, etree._ElementTree]



def write(
	source: typing.Union[str, bytes, ElementTypes],
	dest: Path,
	parser=None,
	logger=None,
	pretty_print=False,
	xml_declaration=True
):
	assert source is not None
	assert isinstance(source, (str, bytes, etree.ElementBase, etree._Element, etree._ElementTree))
	assert dest is not None
	assert dest
	dest = coerce_path(dest)

	if isinstance(source, (str, bytes)):
		source = read(source, parser=parser, logger=logger)

	tree = etree.ElementTree(source)
	tree.write(
		str(dest),  #
		encoding=r'utf-8',
		xml_declaration=xml_declaration,
		pretty_print=pretty_print
	)



#!/usr/bin/env python3
# This file is a part of marzer/poxy and is subject to the the terms of the MIT license.
# Copyright (c) Mark Gillard <mark.gillard@outlook.com.au>
# See https://github.com/marzer/poxy/blob/master/LICENSE for the full license text.
# SPDX-License-Identifier: MIT

try:
	from . import main as m
except ImportError:
	import poxy.main as m

if __name__ == '__main__':
	m.main()
